/*
 *  FSPlugInObject.h
 *  FlareStorm
 *
 *  Created by 2510 on Wed May 07 2003.
 *  Copyright (c) 2003-2004 by 2510. All rights reserved.
 *
 */

/*
 *  Version 1.4 : Feb 24 2004
 *   FSCoreInterfaceMessage_Notification を追加
 *   plugin_execcomplete を追加
 *
 *  Version 1.3 : Jan 11 2004
 *   FSCoreInterfaceMessage_Group を追加
*/

#ifndef included_FSPlugInObject
#define included_FSPlugInObject

#ifndef defined_FSPlugInObject
#define defined_FSPlugInObject
typedef struct FSPlugInObject FSPlugInObject;
#endif

#ifndef defined_FSCore
#define defined_FSCore
typedef struct FSCore FSCore;
#endif

enum
{
	FSCorePhase_Init = 0,
	FSCorePhase_Override,
	FSCorePhase_Connect,
	FSCorePhase_InitDep,
	FSCorePhase_Complete
};

enum
{
	/* 2 - 65535		: システム非依存のメッセージ */
	/* 65536 - int.max	: システム依存の拡張メッセージ */
	
	/* エラーコード(指定が無い場合の返り値)
		0:成功
		-1:使用不可
	*/
	
	/* ABOUT: About 画面を表示 */
	/* return: 0:成功 1:失敗 */
	/* object: (なし) */
	FSCoreInterfaceMessage_About = 0,
	
	/* CONFIGURE: 設定画面を表示 */
	/* return: 0:成功 1:失敗 */
	/* object: (なし) */
	FSCoreInterfaceMessage_Configure,
	
	/* GROUP: プラグインのグループ名 */
	/* return: 常に 0 */
	/* object: [out] (const char *) グループ名 */
	/* FSPlugInGroup.h を参照 */
	FSCoreInterfaceMessage_Group,
	
	/* NOTIFICATION: 通知 */
	/* return: 常に 0 */
	/* object: [in] (const char *) 通知の名前 */
	/* FSPlugInNotification.h を参照 */
	FSCoreInterfaceMessage_Notification,
	
	/* RESPONDS TO: メッセージが使用可能か調べる */
	/* object: [in] unsigned long *message */
	/* 返り値: 0:使用不可 1:使用可能 */
	FSCoreInterfaceMessage_RespondsTo = 65535,
};

#ifndef defined_FSPlugInObject_struct
#define defined_FSPlugInObject_struct
struct FSPlugInObject
{
	/* @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ */
	/* for plug-in activity controls : max. 8 operations */
	
	/* plugin_open: エミュレータの実行開始 */
	/*  phase 0. INIT: 処理を開始 */
	/*  phase 1. OVERRIDE: 他のプラグインの処理の上書き */
	/*  phase 2. CONNECT: 他のプラグインの処理への接続 */
	/*  phase 3. INIT_DEPEND: 他のプラグインに依存した初期化 */
	/*  phase 4. COMPLETE: 完了処理 */
	void	(*plugin_open)(void *plugin,int phase);
	
	/* plugin_open: エミュレータの実行終了 */
	void	(*plugin_close)(void *plugin);
	
	/* plugin_reset: リセット要求 */
	/*	全てのデータの初期化 */
	void	(*plugin_reset)(void *plugin);
	
	/* plugin_attach: サービス登録 */
	/*	phase 0. INIT: 他のプラグインへの接続解除, サービスの登録 */
	/*  phase 1. OVERRIDE: サービスのオーバーライド */
	/*	phase 2. CONNECT: 他のプラグインのサービスへの接続(使用は禁止) */
	/*	phase 3. INIT_DEPEND: 他のプラグインに依存した初期化 */
	/*  phase 4. COMPLETE: 完了、インターフェース起動 */
	void	(*plugin_attach)(void *plugin,FSCore *core,int phase);
	
	/* plugin_async: async のコールバック */
	void	(*plugin_async)(void *plugin,int cycle,unsigned long message);
	
	/* plugin_execcomplete: FSCoreExecuteFunction のコールバック */
	void	(*plugin_execcomplete)(void *plugin,unsigned long info1,unsigned long info2,unsigned long info3);
	
	void	*plugin_reserved0[2];
	
	/* @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ */
	/* for plug-in managements : max. 8 operations */
	
	/* plugin_identifier: プラグインの識別子 */
	const char *(*plugin_identifier)(void *plugin);
	
	/* plugin_description: 簡単な説明文(US-ASCII) */
	const char *(*plugin_description)(void *plugin);
	
	/* plugin_freeze: フリーズデータの生成または解凍 */
	/*  freeze が真ならデータを生成、偽なら *data から解凍 */
	int		(*plugin_freeze)(void *plugin,void **data,unsigned *length,int freeze);
	
	/* plugin_interface: message に応じたインターフェースの操作をする */
	int		(*plugin_interface)(void *plugin,unsigned long message,void **object);
	
	void	*plugin_reserved1[4];
};
#endif

#endif