/*
 *  PSRegister.h
 *  PlayStation Processor Register numbering
 */

#ifndef included_PSRegister
#define included_PSRegister

enum
{
	// CPU
	PSRegIndexCPU = 0,
	PSRegZero = 0,		// zero
	PSRegAT = 1,		// assembler temporary
	PSRegV0 = 2,
	PSRegV1 = 3,
	PSRegA0 = 4,		// argument 0
	PSRegA1 = 5,		// argument 1
	PSRegA2 = 6,		// argument 2
	PSRegA3 = 7,		// argument 3
	PSRegT0 = 8,		// temporary 0
	PSRegT1 = 9,		// temporary 1
	PSRegT2 = 10,		// temporary 2
	PSRegT3 = 11,		// temporary 3
	PSRegT4 = 12,		// temporary 4
	PSRegT5 = 13,		// temporary 5
	PSRegT6 = 14,		// temporary 6
	PSRegT7 = 15,		// temporary 7
	PSRegS0 = 16,		// saved 0
	PSRegS1 = 17,		// saved 1
	PSRegS2 = 18,		// saved 2
	PSRegS3 = 19,		// saved 3
	PSRegS4 = 20,		// saved 4
	PSRegS5 = 21,		// saved 5
	PSRegS6 = 22,		// saved 6
	PSRegS7 = 23,		// saved 7
	PSRegT8 = 24,		// temporary 8
	PSRegT9 = 25,		// temporary 9
	PSRegK0 = 26,		// kernel 0
	PSRegK1 = 27,		// kernel 1
	PSRegGP = 28,		// global pointer
	PSRegSP = 29,		// stack pointer
	PSRegFP = 30,		// frame pointer
	PSRegRA = 31,		// return address
	PSRegHI = 32,		// multiply result high word
	PSRegLO = 33,		// multiply result low word
	PSRegPC = 34,		// program counter
	
	// SCP
	PSRegIndexSCP = 35,
	PSRegSR = 12,		// status
	PSRegCAUSE = 13,	// exception cause
	PSRegEPC = 14,		// exception program counter
	
	// GTE
	PSRegIndexGTE = 67,
	PSRegGTEControl = 0,
	PSRegGTEData = 32,
	
	PSRegCount = 32 + 3 + 32 + 64
};

typedef enum
{
	PSRegR1112 = 0,
	PSRegR1321 = 1,
	PSRegR2223 = 2,
	PSRegR3132 = 3,
	PSRegR33 = 4,
	PSRegTRX = 5,
	PSRegTRY = 6,
	PSRegTRZ = 7,
	PSRegL1112 = 8,
	PSRegL1321 = 9,
	PSRegL2223 = 10,
	PSRegL3132 = 11,
	PSRegL33 = 12,
	PSRegRBK = 13,
	PSRegGBK = 14,
	PSRegBBK = 15,
	PSRegLR1R2 = 16,
	PSRegLR3G1 = 17,
	PSRegLG2G3 = 18,
	PSRegLB1B2 = 19,
	PSRegLB3 = 20,
	PSRegRFC = 21,
	PSRegGFC = 22,
	PSRegBFC = 23,
	PSRegOFX = 24,
	PSRegOFY = 25,
	PSRegH = 26,
	PSRegDQA = 27,
	PSRegDQB = 28,
	PSRegZSF3 = 29,
	PSRegZSF4 = 30,
	PSRegFLAG = 31,
	
	PSRegVX0VY0 = 0,
	PSRegVZ0 = 1,
	PSRegVX1VY1 = 2,
	PSRegVZ1 = 3,
	PSRegVX2VY2 = 4,
	PSRegVZ2 = 5,
	PSRegBGR = 6,
	PSRegOTZ = 7,
	PSRegIR0 = 8,
	PSRegIR1 = 9,
	PSRegIR2 = 10,
	PSRegIR3 = 11,
	PSRegSXY0 = 12,
	PSRegSXY1 = 13,
	PSRegSXY2 = 14,
	PSRegSXYP = 15,
	PSRegSZ0 = 16,
	PSRegSZ1 = 17,
	PSRegSZ2 = 18,
	PSRegSZ3 = 19,
	PSRegRGB0 = 20,
	PSRegRGB1 = 21,
	PSRegRGB2 = 22,
	PSRegRGB3 = 23,
	PSRegMAC0 = 24,
	PSRegMAC1 = 25,
	PSRegMAC2 = 26,
	PSRegMAC3 = 27,
	PSRegIRGB = 28,
	PSRegORGB = 29,
	PSRegLZCS = 30,
	PSRegLZCR = 31
}	PSGTEReg;

typedef struct PSGTE
{
	// Control
	signed short	r12,r11,		// 0
					r21,r13,		// 1
					r23,r22,		// 2
					r32,r31,		// 3
					pd0,r33;		// 4
#if !__cplusplus
	signed long		trx,			// 5
					try,			// 6
					trz;			// 7
#else
	signed long		tr_x,			// 5
					tr_y,			// 6
					tr_z;			// 7
#endif
	signed short	l12,l11,		// 8
					l21,l13,		// 9
					l23,l22,		// 10
					l32,l31,		// 11
					pd1,l33;		// 12
	signed long		rbk,			// 13
					gbk,			// 14
					bbk;			// 15
	signed short	lr2,lr1,		// 16
					lg1,lr3,		// 17
					lg3,lg2,		// 18
					lb2,lb1,		// 19
					pd2,lb3;		// 20
	signed long		rfc,			// 21
					gfc,			// 22
					bfc,			// 23
					ofx,			// 24
					ofy;			// 25
	unsigned short	pd3,h;			// 26
	signed long		dqa,			// 27
					dqb;			// 28
	signed short	pd4,zsf3,		// 29
					pd5,zsf4;		// 30
	unsigned long	flag;			// 31
	
	// Data
	signed short	vy0,vx0,		// 0
					pd6,vz0,		// 1
					vy1,vx1,		// 2
					pd7,vz1,		// 3
					vy2,vx2,		// 4
					pd8,vz2;		// 5
	unsigned char	code,b,g,r;		// 6
	unsigned long	otz;			// 7
	signed long		ir0,			// 8
					ir1,			// 9
					ir2,			// 10
					ir3;			// 11
	signed short	sy0,sx0,		// 12
					sy1,sx1,		// 13
					sy2,sx2,		// 14
					syp,sxp;		// 15
	unsigned short	pd9,sz0,		// 16
					pdA,sz1,		// 17
					pdB,sz2,		// 18
					pdC,sz3;		// 19
	unsigned long	rgb0,			// 20
					rgb1,			// 21
					rgb2,			// 22
					res1;			// 23
	signed long		mac0,			// 24
					mac1,			// 25
					mac2,			// 26
					mac3,			// 27
					irgb,			// 28
					orgb,			// 29
					lzcs,			// 30
					lzcr;			// 31
}	PSGTE;

typedef struct PSGTE_ShortVector
{
	short		y,x,pad,z;
}	PSGTE_ShortVector;

typedef struct PSGTE_LongVector
{
	long		x,y,z;
}	PSGTE_LongVector;

typedef struct PSGTE_Matrix
{
	short		m12,m11,
				m21,m13,
				m23,m22,
				m32,m31,
				pad,m33;
}	PSGTE_Matrix;

#define PSGTE_Vector0(gte) ((PSGTE_ShortVector *)&(((PSGTE *)(gte))->vy0))
#define PSGTE_Vector1(gte) ((PSGTE_ShortVector *)&(((PSGTE *)(gte))->vy1))
#define PSGTE_Vector2(gte) ((PSGTE_ShortVector *)&(((PSGTE *)(gte))->vy2))

#define PSGTE_Translation(gte) ((PSGTE_LongVector *)&(((PSGTE *)(gte))->trx))
#define PSGTE_FarColor(gte) ((PSGTE_LongVector *)&(((PSGTE *)(gte))->rfc))
#define PSGTE_BackgroundColor(gte) ((PSGTE_LongVector *)&(((PSGTE *)(gte))->rbk))

#define PSGTE_RotationMatrix(gte) ((PSGTE_Matrix *)&(((PSGTE *)(gte))->r12))
#define PSGTE_LightMatrix(gte) ((PSGTE_Matrix *)&(((PSGTE *)(gte))->l12))
#define PSGTE_ColorMatrix(gte) ((PSGTE_Matrix *)&(((PSGTE *)(gte))->lr2))

#endif