/*
 *  corelink.h
 *  FlareStorm: libCorelink
 *
 *  Created by 2510 on Thu Apr 24 2003.
 *  Copyright (c) 2003-2004 by 2510. All rights reserved.
 *
 */

#ifndef included_corelink
#define included_corelink

#include "FSBase.h"
#include "PSException.h"

#include <stdarg.h>

#ifndef defined_FSCore
#define defined_FSCore
typedef struct FSCore FSCore;
#endif

#ifndef defined_FSASyncTaskID
#define defined_FSASyncTaskID
typedef unsigned int FSAsyncTaskID;
enum
{
	FSAsyncTaskIDNULL = 0xFFFFFFFF
};
#endif

#ifndef defined_FSExecution
#define defined_FSExecution
typedef struct FSExecution FSExecution;
#endif

#ifndef defined_FSPlugIn
#define defined_FSPlugIn
typedef struct FSPlugIn FSPlugIn;
#endif

#ifndef defined_FSMemoryCore
#define defined_FSMemoryCore
typedef struct FSMemoryCore FSMemoryCore;
#endif

#ifndef defined_FSAddressTrapManager
#define defined_FSAddressTrapManager
typedef struct FSAddressTrapManager FSAddressTrapManager;
#endif

#ifndef defined_FSAddressTrap
#define defined_FSAddressTrap
typedef struct FSAddressTrap FSAddressTrap;
#endif

#ifndef defined_FSAddressTrap_struct
#define defined_FSAddressTrap_struct
struct FSAddressTrap
{
	mips_address	address_physical;
	mips_address	(*callback)(void *context,FSExecution *execution_data,mips_address address_logical);
	void			*context;
	int				act;
};

#ifndef defined_FSThread
#define defined_FSThread
typedef struct FSThread FSThread;
#endif

#ifndef defined_FSThreadTaskID
#define defined_FSThreadTaskID
typedef struct FSThreadTaskID FSThreadTaskID;
#endif

enum
{
	FSAddressTrapActDynamicBranch = 0
};
#endif

#ifndef defined_FSDataDictionary
#define defined_FSDataDictionary
typedef struct FSDataDictionary FSDataDictionary;
#endif

enum
{
	LibFSCoreLogLevelPrint = 0,
	LibFSCoreLogLevelLog,
	LibFSCoreLogLevelWarning
};

#if __cplusplus
extern "C" {
#endif

void					corelink(void);

/* FSCore */
extern FSCore *			(*LibFSCoreAlloc)(void);
extern void				(*LibFSCoreDealloc)(FSCore *core);

extern void				(*LibFSCoreRun)(FSCore *core);
extern void				(*LibFSCoreStop)(FSCore *core);
extern void				(*LibFSCorePause)(FSCore *core);
extern void				(*LibFSCoreUnpause)(FSCore *core);
extern void				(*LibFSCoreReset)(FSCore *core,mips_address start_logical,mips_address initial_stack);
extern void				(*LibFSCoreStepInstruction)(FSCore *core);

extern void				(*LibFSCoreFreeze)(FSCore *core,void **data,unsigned *length);
extern void				(*LibFSCoreDefrost)(FSCore *core,const void *data,unsigned length);

extern void				(*LibFSCoreLock)(FSCore *core,int stop);
extern void				(*LibFSCoreUnlock)(FSCore *core);

extern void *			(*LibFSCoreLoadPlugIn)(FSCore *core,const char *path,const char *dylib_path);
extern int				(*LibFSCoreUnloadPlugIn)(FSCore *core,void *plugin);
extern int				(*LibFSCoreRestartPlugIns)(FSCore *core);

/*  load:  int hwreg(void *context,int address) */
/* store: void hwreg(void *context,int address,int value) */
extern void				(*LibFSCoreTrapHWReg)(FSCore *core,int address,int size,int store,void *function,void *context);
extern void				(*LibFSCoreGetHWRegTrap)(FSCore *core,int address,int size,int store,void **function,void **context);
/* void systemservice(void *context,int class_code,int function_code) */
extern void				(*LibFSCoreTrapSystemService)(FSCore *core,int class_code,int function_code,void *function,void *context);
extern void				(*LibFSCoreGetSystemService)(FSCore *core,int class_code,int function_code,void **function,void **context);
extern FSAsyncTaskID	(*LibFSCoreRegisterAsync)(FSCore *core,int after,void *plugin,unsigned long message);
extern FSAsyncTaskID	(*LibFSCoreRegisterAsyncInterface)(FSCore *core,int after,void *plugin,unsigned long message);
extern void				(*LibFSCoreCancelAsync)(FSCore *core,void *plugin,FSAsyncTaskID taskID);

extern void				(*LibFSCoreException)(FSCore *core,PSException exception);
extern void				(*LibFSCoreInterrupt)(FSCore *core,PSInterrupt interrupt);
extern void				(*LibFSCoreDMAInterrupt)(FSCore *core,PSDMA dma);
extern void				(*LibFSCoreLockInterrupt)(FSCore *core,PSInterrupt interrupt);
extern void				(*LibFSCoreUnlockInterrupt)(FSCore *core,PSInterrupt interrupt);
extern unsigned long	(*LibFSCoreGetClock)(FSCore *core);

extern int				(*LibFSCoreRegisterService)(FSCore *core,const char *service_id,void *data);
extern void *			(*LibFSCoreGetService)(FSCore *core,const char *service_id);

extern void				(*LibFSCoreLoadImage)(FSCore *core,const void *image,mips_address logical_start,unsigned length);
extern void				(*LibFSCoreFlushInstructionCache)(FSCore *core,mips_address physical_start,unsigned length);

extern int				(*LibFSCoreExecuteFunction)(FSCore *core,mips_address logical_start);
//extern void				(*LibFSCoreExecuteFunction)(FSCore *core,mips_address start_pc,void *plugin,unsigned long info1,unsigned long info2,unsigned long info3);
extern void				(*LibFSCoreExitRecursiveExecution)(FSCore *core);

extern void				(*LibFSCorePrint)(FSCore *core,int level,const char *format,...); /* UTF-8 */
extern void				(*LibFSCoreVprint)(FSCore *core,int level,const char *format,va_list args); /* UTF-8 */
extern void				(*LibFSCorePrintVersion)(FSCore *core);
extern void				(*LibFSCoreCaptureScreen)(FSCore *core,void **bitmap,int *is24bit,int *rowbytes,int *width,int *height);

extern void				(*LibFSCoreConfigure)(FSCore *core);

extern FSMemoryCore *			(*LibFSCoreGetMemoryCore)(FSCore *core);
extern FSExecution *			(*LibFSCoreGetExecutionData)(FSCore *core);
extern FSAddressTrapManager *	(*LibFSCoreGetAddressTrapManager)(FSCore *core);
extern FSThread *				(*LibFSCoreGetThread)(FSCore *core);

/* FSPlugIn */
extern FSPlugIn *		(*LibFSPlugInAlloc)(const char *path,const char *dylib_path);
extern void				(*LibFSPlugInDealloc)(FSPlugIn *plugin);
extern void *			(*LibFSPlugInGetObject)(FSPlugIn *plugin);

/* FSMemoryCore */
extern mips_address		(*LibFSMemoryCoreGetPhysical)(FSMemoryCore *core,mips_address address_logical);
extern mips_address		(*LibFSMemoryCoreGetLogical)(FSMemoryCore *core,mips_address address_physical);
extern void *			(*LibFSMemoryCoreImageAddress)(FSMemoryCore *core);
extern void *			(*LibFSMemoryCoreStaticROMAlloc)(FSMemoryCore *core,mips_address start,mips_address end);
extern void				(*LibFSMemoryCoreDynamicKernelAlloc)(FSMemoryCore *core,void *plugin,const char *identifier,unsigned size,mips_address *result);

/* FSAddressTrapManager */
extern void				(*LibFSAddressTrapManagerAddTrap)(FSAddressTrapManager *manager,FSAddressTrap *trapToAdd);
extern void				(*LibFSAddressTrapManagerRemoveTrap)(FSAddressTrapManager *manager,mips_address trap_address_physical);

extern FSThreadTaskID *	(*LibFSThreadExecute)(FSThread *thread,void (*function)(void *context),void *context);
extern void				(*LibFSThreadJoin)(FSThread *thread,FSThreadTaskID *task);

/* FSDataDictionary */
extern unsigned long	(*LibFSDataDictionaryLoadWord)(const void *ptr);
extern void				(*LibFSDataDictionaryStoreWord)(void *ptr,mips_word data);

extern FSDataDictionary *(*LibFSDataDictionaryAlloc)(void);
extern void				(*LibFSDataDictionaryInitWithData)(FSDataDictionary *dictionary,const void *data,unsigned length);
extern void				(*LibFSDataDictionaryDealloc)(FSDataDictionary *dictionary);
extern void				(*LibFSDataDictionaryGetDataAndDealloc)(FSDataDictionary *dictionary,void **data,unsigned *length);

extern void				(*LibFSDataDictionaryAppend)(FSDataDictionary *dictionary,const char *key,const void *data,unsigned length);
extern void				(*LibFSDataDictionaryNextData)(FSDataDictionary *dictionary,const char **key,const void **data,unsigned *length);
extern void				(*LibFSDataDictionaryDataForKey)(FSDataDictionary *dictionary,const char *key,const void **data,unsigned *length);
extern int				(*LibFSDataDictionaryReadDataForKey)(FSDataDictionary *dictionary,const char *key,void *data,unsigned length);

extern void				(*LibFSDataDictionaryGetData)(FSDataDictionary *dictionary,const void **data,unsigned *length);

/* Util. */
extern void				(*LibFSVMUtilMakeExecutable)(void *start,void *end);

#if __cplusplus
}
#endif

#endif